package com.four.auth.service;

import com.four.auth.form.RegisterBody;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import com.four.common.core.constant.CacheConstants;
import com.four.common.core.constant.Constants;
import com.four.common.core.constant.SecurityConstants;
import com.four.common.core.constant.UserConstants;
import com.four.common.core.domain.R;
import com.four.common.core.enums.UserStatus;
import com.four.common.core.exception.ServiceException;
import com.four.common.core.text.Convert;
import com.four.common.core.utils.StringUtils;
import com.four.common.core.utils.ip.IpUtils;
import com.four.common.redis.service.RedisService;
import com.four.common.security.utils.SecurityUtils;
import com.four.system.api.RemoteUserService;
import com.four.system.api.domain.SysUser;
import com.four.system.api.model.LoginUser;
import com.four.common.duck.request.RequestRegistrationInformation;

import java.math.BigDecimal;

/**
 * 登录校验方法
 *
 * @author ruoyi
 */
@Component
@SuppressWarnings("all")
public class SysLoginService
{
    @Autowired
    private RemoteUserService remoteUserService;

    @Autowired
    private SysPasswordService passwordService;

    @Autowired
    private SysRecordLogService recordLogService;

    @Autowired
    private RedisService redisService;

    /**
     * 登录
     */
    public LoginUser login(String email, String password)
    {
        // 用户名或密码为空 错误
        if (StringUtils.isAnyBlank(email, password))
        {
            recordLogService.recordLogininfors(email, Constants.LOGIN_FAIL, "用户/密码必须填写");
            throw new ServiceException("用户/密码必须填写");
        }
        // 密码如果不在指定范围内 错误
        if (password.length() < UserConstants.PASSWORD_MIN_LENGTH
                || password.length() > UserConstants.PASSWORD_MAX_LENGTH)
        {
            recordLogService.recordLogininfors(email, Constants.LOGIN_FAIL, "用户密码不在指定范围");
            throw new ServiceException("用户密码不在指定范围");
        }
        // 用户名不在指定范围内 错误
        if (email.length() < UserConstants.USERNAME_MIN_LENGTH
                || email.length() > UserConstants.USERNAME_MAX_LENGTH)
        {
            recordLogService.recordLogininfors(email, Constants.LOGIN_FAIL, "用户名不在指定范围");
            throw new ServiceException("用户名不在指定范围");
        }
        // IP黑名单校验
        String blackStr = Convert.toStr(redisService.getCacheObject(CacheConstants.SYS_LOGIN_BLACKIPLIST));
        if (IpUtils.isMatchedIp(blackStr, IpUtils.getIpAddr()))
        {
            recordLogService.recordLogininfors(email, Constants.LOGIN_FAIL, "很遗憾，访问IP已被列入系统黑名单");
            throw new ServiceException("很遗憾，访问IP已被列入系统黑名单");
        }
        // 查询用户信息
        R<LoginUser> userResult = remoteUserService.getUserInfoByEmail(email, SecurityConstants.INNER);

        if (StringUtils.isNull(userResult) || StringUtils.isNull(userResult.getData()))
        {
            recordLogService.recordLogininfors(email, Constants.LOGIN_FAIL, "登录用户不存在");
            throw new ServiceException("登录用户：" + email + " 不存在");
        }

        if (R.FAIL == userResult.getCode())
        {
            throw new ServiceException(userResult.getMsg());
        }

        LoginUser userInfo = userResult.getData();
        SysUser user = userResult.getData().getSysUser();
        if (UserStatus.DELETED.getCode().equals(user.getDelFlag()))
        {
            recordLogService.recordLogininfors(email, Constants.LOGIN_FAIL, "对不起，您的账号已被删除");
            throw new ServiceException("对不起，您的账号：" + email + " 已被删除");
        }

        if (UserStatus.DISABLE.getCode().equals(user.getStatus()))
        {
            recordLogService.recordLogininfors(email, Constants.LOGIN_FAIL, "用户已停用，请联系管理员");
            throw new ServiceException("对不起，您的账号：" + email + " 已停用");
        }
        passwordService.validate(user, password);
        recordLogService.recordLogininfors(email, Constants.LOGIN_SUCCESS, "登录成功");
        return userInfo;
    }

    public void logout(String loginName)
    {
        recordLogService.recordLogininfor(loginName, Constants.LOGOUT, "退出成功");
    }

    /**
     * 注册
     */
    public void register(String userName, String password)
    {
        // 用户名或密码为空 错误
        if (StringUtils.isAnyBlank(userName, password))
        {
            throw new ServiceException("用户/密码必须填写");
        }
        if (userName.length() < UserConstants.USERNAME_MIN_LENGTH
                || userName.length() > UserConstants.USERNAME_MAX_LENGTH)
        {
            throw new ServiceException("账户长度必须在2到20个字符之间");
        }
        if (password.length() < UserConstants.PASSWORD_MIN_LENGTH
                || password.length() > UserConstants.PASSWORD_MAX_LENGTH)
        {
            throw new ServiceException("密码长度必须在5到20个字符之间");
        }

        // 注册用户信息
        SysUser sysUser = new SysUser();
        sysUser.setUserName(userName);
        sysUser.setNickName(userName);
        sysUser.setPassword(SecurityUtils.encryptPassword(password));
        R<?> registerResult = remoteUserService.registerUserInfo(sysUser, SecurityConstants.INNER);

        if (R.FAIL == registerResult.getCode())
        {
            throw new ServiceException(registerResult.getMsg());
        }
        recordLogService.recordLogininfor(userName, Constants.REGISTER, "注册成功");
    }


    /***
     * 用户名  密码登录
     * @param userName
     * @param password
     * @return
     */

    public LoginUser userLogin(String userName, String password) {

        if(StringUtils.isAllBlank(userName,password)){
            recordLogService.recordLogininfor(userName,Constants.LOGIN_FAIL,"用户、密码必须填写");
            throw new ServiceException("用户/密码必须填写");
        }

        if(password.length() < UserConstants.PASSWORD_MIN_LENGTH
                ||password.length()> UserConstants.PASSWORD_MAX_LENGTH){
            recordLogService.recordLogininfor(userName,Constants.LOGIN_FAIL,"用户密码不在指定范围");
            throw new ServiceException("用户密码不在指定范围");
        }

        if(userName .length() < UserConstants.USERNAME_MIN_LENGTH || userName.length() > UserConstants.PASSWORD_MAX_LENGTH){
            recordLogService.recordLogininfor(userName, Constants.LOGIN_FAIL,"用户不在指定范围");
            throw new ServiceException("用户不在指定范围");
        }

        String blackStr = Convert.toStr(redisService.getCacheObject(CacheConstants.SYS_LOGIN_BLACKIPLIST));
        if(IpUtils.isMatchedIp(blackStr,IpUtils.getIpAddr())){
            recordLogService.recordLogininfor(blackStr,Constants.LOGIN_FAIL,"很遗憾,访问IP已被列入系统黑名单");
            throw new ServiceException("很遗憾,访问IP已被列入系统黑名单");
        }


        R<LoginUser> userResult = remoteUserService.getUserInfo(userName, SecurityConstants.INNER);

        if (StringUtils.isNull(userResult) || StringUtils.isNull(userResult.getData()))
        {
            recordLogService.recordLogininfor(userName, Constants.LOGIN_FAIL, "登录用户不存在");
            throw new ServiceException("登录用户：" + userName + " 不存在");
        }

        if(R.FAIL == userResult.getCode()){
            throw new ServiceException(userResult.getMsg());
        }

        LoginUser dataInfo = userResult.getData();

        SysUser sysUser = userResult.getData().getSysUser();
        if(UserStatus.DISABLE.getCode().equals(sysUser.getDelFlag())){
            recordLogService.recordLogininfor(userName,Constants.LOGIN_FAIL,"对不起,您的账号已被删除");
            throw new ServiceException("对不起,你的账号:"+userName + "已被删除");
        }
        if(UserStatus.DISABLE.getCode().equals(sysUser.getStatus())){
            recordLogService.recordLogininfor(userName,Constants.LOGIN_FAIL,"用户已停用,请联系管理员");
            throw new ServiceException("对不起 您的账号:"+userName + "已停用");
        }
        passwordService.validate(sysUser,password);
        recordLogService.recordLogininfor(userName,Constants.LOGIN_SUCCESS,"登录成功");
        return dataInfo;
    }


    /**
     * 医生注册
     * @param registrationInformation
     */
    public void healthRegister(RequestRegistrationInformation requestRegistrationInformation) {

        if(StringUtils.isAllBlank(requestRegistrationInformation.getRealName(),requestRegistrationInformation.getPassword())){
            throw new ServiceException("用户/密码必须填写");
        }
        if(requestRegistrationInformation.getPassword().length() < UserConstants.PASSWORD_MIN_LENGTH
                || requestRegistrationInformation.getPassword().length()> UserConstants.PASSWORD_MAX_LENGTH){
            throw new ServiceException("密码长度必须在5到20个字符");
        }

        requestRegistrationInformation.setPassword(SecurityUtils.encryptPassword(requestRegistrationInformation.getPassword()));

        SysUser sysUser = new SysUser();
        sysUser.setDeptId(3L);
        sysUser.setBindWechatStatus(1);
        sysUser.setRealNameAuthenticationStatus(1);
        sysUser.setBindBankCardStatus(1);
        sysUser.setDifferentialState(2);
        sysUser.setStatus("0");
        sysUser.setUserMoney(new BigDecimal(0));
        sysUser.setEmail(requestRegistrationInformation.getEmail());
        sysUser.setNickName(requestRegistrationInformation.getRealName());
        sysUser.setPassword(SecurityUtils.encryptPassword(requestRegistrationInformation.getPassword()));
        sysUser.setUserName(requestRegistrationInformation.getRealName());


        System.out.println(sysUser);

        R<?> registerResult = remoteUserService.healthRegister(sysUser, SecurityConstants.INNER);

        if(R.FAIL == registerResult.getCode()){
            throw new ServiceException(registerResult.getMsg());
        }
        recordLogService.recordLogininfor(requestRegistrationInformation.getEmail(), Constants.REGISTER,"注册成功");
    }

    /***
     * 用户注册
     * @param registerBody
     */
    public void PatientRegisterBody(RegisterBody registerBody) {
        if(StringUtils.isAllBlank(registerBody.getEmail(),registerBody.getPassword())){
            throw new ServiceException("邮箱/密码必须填写");
        }
        if(registerBody.getPassword().length() < UserConstants.PASSWORD_MIN_LENGTH
            || registerBody.getPassword().length() > UserConstants.PASSWORD_MAX_LENGTH){

            throw new ServiceException("密码必须填写");
        }

        registerBody.setPassword(SecurityUtils.encryptPassword(registerBody.getPassword()));


        SysUser sysUser = new SysUser();
        sysUser.setDeptId(4L);
        sysUser.setBindBankCardStatus(1);
        sysUser.setRealNameAuthenticationStatus(1);
        sysUser.setBindBankCardStatus(1);
        sysUser.setDifferentialState(2);
        sysUser.setStatus("0");
        sysUser.setUserMoney(new BigDecimal(0));
        sysUser.setEmail(registerBody.getEmail());
        sysUser.setNickName(registerBody.getEmail());
        sysUser.setUserName(registerBody.getEmail());
        sysUser.setPassword(registerBody.getPassword());


        System.out.println(sysUser);

        R<?> registerBodys = remoteUserService.PatientRegisterBody(sysUser,SecurityConstants.INNER);
        if(R.FAIL == registerBodys.getCode()){
            throw new ServiceException(registerBodys.getMsg());
        }
        recordLogService.recordLogininfor(registerBody.getEmail(),Constants.REGISTER,"注册成功");
    }

    /***
     * 人脸邮箱后台登录
     * @param email
     * @return
     */
    public LoginUser emailLogin(String email) {
        if(StringUtils.isAllBlank(email)){
            recordLogService.recordEmailLoginFors(email,Constants.LOGIN_FAIL,"邮箱必须填写");
            throw new ServiceException("邮箱必须填写");
        }

        if(email.length() < UserConstants.USERNAME_MIN_LENGTH || email.length() > UserConstants.USERNAME_MAX_LENGTH){
            recordLogService.recordEmailLoginFors(email,Constants.LOGIN_FAIL,"用户不在指定范围内");
            throw new ServiceException("用户不在指定范围");

        }

        String blackStr = Convert.toStr(redisService.getCacheObject(CacheConstants.SYS_LOGIN_BLACKIPLIST));
        if(IpUtils.isMatchedIp(blackStr,IpUtils.getIpAddr())){
            recordLogService.recordEmailLoginFors(blackStr,Constants.LOGIN_FAIL,"很遗憾,访问IP已被列入系统黑名单");
            throw new ServiceException("很遗憾,访问IP已被列入系统黑名单");
        }

        R<LoginUser> userResult = remoteUserService.getUserInfo(email, SecurityConstants.INNER);
        if(StringUtils.isNull(userResult) || StringUtils.isNull(userResult.getData())){
            recordLogService.recordEmailLoginFors(email ,Constants.LOGIN_FAIL,"登录用户不存在");
            throw new ServiceException("登录用户:" + email + "不存在");
        }
        if(R.FAIL == userResult.getCode()){
            throw new ServiceException(userResult.getMsg());
        }

        LoginUser dataInfo = userResult.getData();
        SysUser sysUser = userResult.getData().getSysUser();
        if(UserStatus.DISABLE.getCode().equals(sysUser.getDelFlag())){
            recordLogService.recordEmailLoginFors(email,Constants.LOGIN_FAIL,"对不起,您的账号已被删除");
            throw new ServiceException("对不起,你的账号" + email + "已被删除");
        }
        if(UserStatus.DISABLE.getCode().equals(sysUser.getStatus())){
            recordLogService.recordEmailLoginFors(email,Constants.LOGIN_FAIL,"用户已停用,");
        }
        recordLogService.recordEmailLoginFors(email,Constants.LOGIN_SUCCESS,"登录成功");
        return dataInfo;
    }
}
